@@ -91,6 +91,21 @@ def runscript(script,isansible=True):
 
 PLEDGEING_INTEVERAL=150#s
 
+# in: --resourceUsage 0.4
+# in: --resourceUsage=0.4
+# out 0.4
+def extractValue(line,key="--resourceUsage"):
+    splited=line.split()
+    for idx,slice in enumerate(splited):
+        if key not in slice: continue
+        if key == slice: return splited[idx+1]
+        if "=" in slice: return slice.split("=")[1]
+    return None
+
+import datetime
+
+# export LOTUS_MINER_PATH=
+# 
 def autopledge(interval,iplst):
     # load ips
     ips=load_ips(iplst)
@@ -98,41 +113,57 @@ def autopledge(interval,iplst):
     # map(ip,ansible_name)
     ansible_names=getAnsibleNames(ips)
     if len(ansible_names) < 1: return msg_ansible_needs_workers
-    ip_process_env_tree={}
-    # inquiry
-    # {'172.26.48.134':{
-    #                   '11111': {LOTUS_WORKER_PATH:/filecoin1/lotusworker5}},
-    #                   22222': {LOTUS_WORKER_PATH:/filecoin1/lotusworker1}
-    #                   }
-    # }
-    for ip in ips:
-        ip_process_env_tree[ip]={}
-        script="ansible workername -m shell -a 'ps -ef|grep lotus'".replace('workername',ansible_names[ip])
-        procs = list(filter(None,[line \
-                                for line in runscript(script) \
-                                if 'lotus-worker' in line]))
-        for proc in procs:
-            envdict={}
-            pid=proc.split()[1]
-            # tasks for process
-            envdict["TYPE"] = "COMMIT" if "--precommit1=false" in proc else "PRECOMMIT"
-            # (process,LOTUS_WORKER_PATH)
-            script="ansible {0} -m shell -a 'strings /proc/{1}/environ'".format(ansible_names[ip],pid)
-            envs=list(filter(None,[line\
-                                 for line in runscript(script) \
-                                 if any(filter in line for filter in ['FIL','LOTUS','CPU','CUDA','TMP'])]))
-            # environment variables added to tree
-            for env in envs:
-                k,v=env.split("=")
-                envdict[k]=v
-            ip_process_env_tree[ip][pid]=envdict
-        
     INSPECT_INTERVAL = interpret(interval)
     while 1:
+        print("Time: ",datetime.datetime.now())
+        ip_process_env_tree={}
+        # inquiry
+        # {'172.26.48.134':{
+        #                   '11111': {LOTUS_WORKER_PATH:/filecoin1/lotusworker5}},
+        #                   22222': {LOTUS_WORKER_PATH:/filecoin1/lotusworker1}
+        #                   }
+        # }
+        # make (ip,procs)
+        for ip in ips:
+            ip_process_env_tree[ip]={}
+            script="ansible workername -m shell -a 'ps -ef|grep lotus'".replace('workername',ansible_names[ip])
+            procs = list(filter(None,[line \
+                                    for line in runscript(script) \
+                                    if 'lotus-worker' in line]))
+            resource_for_precommit = 0.0
+            ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"] = 0.0
+            ip_process_env_tree[ip]['CPU'] = {}
+            ip_process_env_tree[ip]['MEM'] = {}
+            for proc in procs:
+                envdict={}
+                pid=proc.split()[1]
+                # tasks for process
+                envdict["TYPE"] = "COMMIT" if "--precommit1=false" in proc else "PRECOMMIT"
+                # resourceUsage
+                if "resourceUsage" in proc:
+                    try: envdict["RESOURCE_USAGE"] = float(extractValue(proc,key="--resourceUsage"))
+                    except: envdict["RESOURCE_USAGE"] = 1.0
+                # (process,LOTUS_WORKER_PATH)
+                script="ansible {0} -m shell -a 'strings /proc/{1}/environ'".format(ansible_names[ip],pid)
+                envs=list(filter(None,[line\
+                                    for line in runscript(script) \
+                                    if any(filter in line for filter in ['FIL','LOTUS','CPU','CUDA','TMP'])]))
+                # environment variables added to tree
+                for env in envs:
+                    k,v=env.split("=")
+                    envdict[k]=v
+                ip_process_env_tree[ip][pid]=envdict
+                # resource usage
+                if envdict["TYPE"] == "PRECOMMIT" and "RESOURCE_USAGE" in envdict.keys():
+                    resource_for_precommit += envdict["RESOURCE_USAGE"]
+            # resource usage
+            ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"] = 1.0 if resource_for_precommit == 0 else resource_for_precommit
+
+        # check availability for pledging    
         pledgeable_cnt = chkavailable(ip_process_env_tree)
         if any("inspect" in arg for arg in sys.argv): break
         import time
-        while pledgeable_cnt>=0: 
+        while pledgeable_cnt>0: 
             print(runscript("lotus-miner sectors pledge",isansible=False))
             pledgeable_cnt-=1
             time.sleep(PLEDGEING_INTEVERAL)
@@ -151,12 +182,15 @@ def chkavailable(ip_process_env_tree):
     jobs=list(filter(None,runscript(script,isansible=False)))
     jstats=set([job.split()[4] for job in jobs if not job.split()[4].isdigit()])
     jstats_precommit="AP|PC1|PC2".split("|")
-    sectors_cnt_assigned = len([job for job in jobs if any(stat in job.split()[1] for stat in jstats_precommit)])
+    sectors_cnt_assigned = len([job for job in jobs if any(stat in job.split()[4] for stat in jstats_precommit)])
     workers = {}
     for job in jobs:
-        if job.split()[2] not in workers.keys():
-            workers[job.split()[2]] = [job.split()[1]]
-        else: workers[job.split()[2]].append(job.split()[1])
+        try: wid= job.split()[2]
+        except: continue
+        if len(wid) !=8 or not all(c in string.hexdigits for c in wid): continue
+        if wid not in workers.keys():
+            workers[wid] = [job.split()[1]]
+        else: workers[wid].append(job.split()[1])
     script='lotus-miner sectors list|egrep "Packing|PreCommit1|PreCommit2|WaitSeed|CommitWait|Committing|FinalizeSector|Removing|RecoveryTimeout"'
     sectors=list(filter(None,runscript(script,isansible=False)))
     sstats=set([sector.split()[1] for sector in sectors if not sector.split()[1].isdigit()])
@@ -171,7 +205,7 @@ def chkavailable(ip_process_env_tree):
     # inspect workers
     print("#######################################################")
     print("WORKER REPORT: ")
-    cached_sectors_cnt=0
+    cached_sectors_cnt,pledgeable_sectors_cnt = 0,0
     for ip,procs in ip_process_env_tree.items():
         
         action = "ansible %s -m shell -a"%ansible_names[ip]
@@ -182,12 +216,15 @@ def chkavailable(ip_process_env_tree):
         # MEM
         output=runscript("ansible workername -m shell -a 'free'".\
                         replace('workername',ansible_names[ip]))[1].split()
-        ip_total_mem = int(float(output[1])/2**20)
-        ip_total_used_mem = int(float(output[2])/2**20)
+        try:
+            ip_total_mem = int(float(output[1])/2**20)
+            ip_total_used_mem = int(float(output[2])/2**20)
+        except: continue
         # CPU
         output=runscript("ansible workername -m shell -a 'nproc --all'".\
                         replace('workername',ansible_names[ip]))[0]
-        ip_total_cpus=int(output)
+        try: ip_total_cpus=int(output)
+        except: continue
         # USED
         output=runscript("ansible workername -m shell -a 'ps -eo pid,%mem,%cpu,comm --sort=-%cpu'".replace('workername',ansible_names[ip]))[1:]
         ip_proc_usage={line.split()[0]:{"mem":line.split()[1],"cpu":line.split()[2],"pname":line.split()[3]} for line in output}
@@ -262,9 +299,9 @@ def chkavailable(ip_process_env_tree):
             ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path][env["TYPE"]]['USED'] +=  ip_process_env_tree[ip][pid]['STORAGE']
             ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path][env["TYPE"]]['CACHED_SECTOR_CNT'] += ip_process_env_tree[ip][pid]['CACHED_SECTOR_CNT']
 
-
             ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]["TYPE"].append(env["TYPE"])
 
+
         # ansible workername -m shell -a 'df'
         df_out=list(filter(None,[line
                                  for line in runscript(action+" 'df'") \
@@ -301,21 +338,25 @@ def chkavailable(ip_process_env_tree):
                 else 0
             ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PLEDGE_USEABLE']=max(0,ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PLEDGE_USEABLE']-PLEDGE_MARGIN)
             ip_process_env_tree[ip]['STORAGE']['PLEDGE_USEABLE(CNT)'] += int(ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PLEDGE_USEABLE']/storage_per_sector_precommit)
-        ip_process_env_tree[ip]['CPU'] = {}
+        # cpu
         ip_process_env_tree[ip]['CPU']['TOTAL'] = ip_total_cpus
         ip_process_env_tree[ip]['CPU']['LOTUS_USED'] = ip_used_cpus
         ip_process_env_tree[ip]['CPU']['NON_LOTUS_USED'] = ip_total_used_cpus-ip_used_cpus
         ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'] = int(max(ip_total_cpus-ip_process_env_tree[ip]['CPU']['NON_LOTUS_USED'],0)/cpus_per_sector)
-        ip_process_env_tree[ip]['MEM'] = {}
+        ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'] = int(min(ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"],1.0)*ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'])
+        # mem
         ip_process_env_tree[ip]['MEM']['TOTAL'] = ip_total_mem
         ip_process_env_tree[ip]['MEM']['LOTUS_USED'] = ip_used_mem
         ip_process_env_tree[ip]['MEM']['NON_LOTUS_USED'] = ip_total_used_mem-ip_used_mem
         ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'] = int(max(ip_total_mem-ip_process_env_tree[ip]['MEM']['NON_LOTUS_USED'],0)/mem_per_sector)
+        ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'] = int(min(ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"],1.0)*ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'])
         # affordability check
-        cached_sectors_cnt+=min(ip_process_env_tree[ip]['STORAGE']['PLEDGE_USEABLE(CNT)'],\
+        pledgeable_sectors_cnt+=min(ip_process_env_tree[ip]['STORAGE']['PLEDGE_USEABLE(CNT)'],\
                                 ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'],\
                                 ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'])
+        # cached sector cnt
+        cached_sectors_cnt+=ip_sector_cnt
 
     import json; print(json.dumps(ip_process_env_tree, indent=4, sort_keys=True))
-    print("CACHED_CNT: {0}, SECTOR_CNT: {1}".format(cached_sectors_cnt,sectors_cnt))
-    return cached_sectors_cnt-sectors_cnt
+    print("PLEDEABLE_CNT: {0}, MINER_SECTOR_CNT: {1}, WORKER_CACHED_SECTOR_CNT: {2}".format(pledgeable_sectors_cnt,sectors_cnt,cached_sectors_cnt))
+    return pledgeable_sectors_cnt-sectors_cnt
